/*
 *  Copyright (C) 2006 DFKI GmbH
 *  Image Understanding and Pattern Recognition Lab
 *  Author: Christian Kofler <christian.kofler@dfki.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "FireWatch.h"

#include <wx/tokenzr.h>

#include <cstdlib>
#include <iostream>
#include <iomanip>

using namespace std;

FireWatch::FireWatch() {
    // default to localhost
    hostname = wxT("localhost");
    defaultPort = 12960;
    
    fileCount = 0;
    resultCount = 0;
    
    fireResults.clear();
    
    setlocale(LC_NUMERIC, "C");
}

FireWatch::~FireWatch() {
    if (!fireResults.empty()) fireResults.clear();
}

void FireWatch::setServer(wxString host, unsigned short port) {
    if(host.IsEmpty()) return;
    
    hostname = host;
    defaultPort = port;
}

bool FireWatch::initialize() {
    bool status = false;
    
    cout << "[FireWatch]  " << "connecting to " << hostname.mb_str();
    cout << " and initializing..." << endl;
    status = client.connect(hostname, defaultPort);
    
    if (status == false) {
        wxMessageBox(wxT("could not connect to ") + hostname );
        return false;
    }
    
    getServerConfig();
    if (path.IsEmpty()) return false;
    
    cout << "[FireWatch]  " << "fileCount: " << fileCount << endl;
    cout << "[FireWatch]  " << "resultCount: " << resultCount << endl;
    cout << "[FireWatch]  " << "path: " << path.mb_str() << endl;

    return status;
}

bool FireWatch::release() {
    bool status = false;
    
    status = client.release();
    if (status) cout << "[FireWatch]  " << "connection closed" << endl;
    else cout << "[FireWatch]  " << "failed to close connection!" << endl;
    
    return status;
}

vector<FireResult>& FireWatch::getRandomImages() {
    wxString imageName;
    
    wxString result = client.request(wxT("random"));
    wxStringTokenizer tokenizer(result);
    
    // clear fireResults if not empty and we have new results
    if (tokenizer.HasMoreTokens() && !fireResults.empty()) fireResults.clear();
    
    // get all result images
    while (tokenizer.HasMoreTokens()) {
        imageName = tokenizer.GetNextToken();
        fireResults.push_back(FireResult(imageName));
    }
    printFireResults();
    
    return fireResults;
}

vector<FireResult>& FireWatch::retrieve(wxString queryImage, bool newfile, bool save) {
    wxString token;
    wxString imageName;
    double score = 0.f;
    wxString query;
    
    if (newfile) {
        query = wxT("newfile");
        if (save) query += wxT("save");
        query += wxT(" ");
    } else {
        query = wxT("retrieve +");
    }
    query +=  queryImage;

    wxString result = client.request(query);
    wxStringTokenizer tokenizer(result);
    
    // clear fireResults if not empty and we have new results
    if (tokenizer.HasMoreTokens() && !fireResults.empty()) fireResults.clear();
    
    // get all result images
    while (tokenizer.HasMoreTokens()) {
        imageName = tokenizer.GetNextToken();
        token = tokenizer.GetNextToken();
        if (!token.IsEmpty()) token.ToDouble((double*)&score);
    
        fireResults.push_back(FireResult(imageName, score));
    }
    printFireResults();
    
    return fireResults;
}

void FireWatch::getServerConfig() {
    wxString token;
    // request server info with "info"
    wxString result = client.request(wxString(wxT("info")));
    wxStringTokenizer tokenizer(result);
    
    // example "info" string from Fire server
    //filelist 1250 results 10 path /data/datasets/otto/OTTO2/fire-data 
    //extensions 0 scoring linear size 2 weight 0 1 weight 1 1 
    //suffix 0 tamura.histo.gz jsd suffix 1 color.histo.gz jsd
    
    // cut "filelist"
    tokenizer.GetNextToken();
    token = tokenizer.GetNextToken();
    if (!token.IsEmpty()) token.ToULong((unsigned long*)&fileCount);
    // cut "results"
    tokenizer.GetNextToken();
    token = tokenizer.GetNextToken();
    if (!token.IsEmpty()) token.ToULong((unsigned long*)&resultCount);
    // cut "path"
    tokenizer.GetNextToken();
    path = tokenizer.GetNextToken();

}

unsigned int FireWatch::setResultCount(unsigned int value) {

    wxString query = wxT("setresults ") + wxString::Format(wxT("%d"), value);
    wxString result = client.request(query);
    
    // cut "results ="
    result = result.After('=');
    if (!result.IsEmpty()) result.ToULong((unsigned long*)&resultCount);    
   
    cout << "[FireWatch] result count = " << resultCount << endl;
    
    return resultCount;
}

void FireWatch::printFireResults() {
    cout << endl << "[FireWatch]  " << "result images: " << endl;
    for (unsigned int i = 0; i < fireResults.size(); ++i) {
        cout << "  name: " << fireResults[i].getName().mb_str();
        cout << " score: " << fireResults[i].getScore() << endl;
    }    
}

